# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1211, 684)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/siviIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setTabShape(QtWidgets.QTabWidget.Triangular)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget_2.setStyleSheet("")
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.filename = QtWidgets.QLabel(self.tab_2)
        self.filename.setObjectName("filename")
        self.horizontalLayout.addWidget(self.filename)
        self.btn_file = QtWidgets.QPushButton(self.tab_2)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/load.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_file.setIcon(icon1)
        self.btn_file.setObjectName("btn_file")
        self.horizontalLayout.addWidget(self.btn_file)
        self.btn_reload = QtWidgets.QPushButton(self.tab_2)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/reload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_reload.setIcon(icon2)
        self.btn_reload.setObjectName("btn_reload")
        self.horizontalLayout.addWidget(self.btn_reload)
        self.btn_close = QtWidgets.QPushButton(self.tab_2)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("rc/close.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_close.setIcon(icon3)
        self.btn_close.setObjectName("btn_close")
        self.horizontalLayout.addWidget(self.btn_close)
        self.verticalLayout_9.addLayout(self.horizontalLayout)
        self.widget = QtWidgets.QWidget(self.tab_2)
        self.widget.setObjectName("widget")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.redraw = QtWidgets.QPushButton(self.widget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/writing.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.redraw.setIcon(icon4)
        self.redraw.setObjectName("redraw")
        self.horizontalLayout_3.addWidget(self.redraw)
        self.btn_preprocessor = QtWidgets.QPushButton(self.widget)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/work.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_preprocessor.setIcon(icon5)
        self.btn_preprocessor.setObjectName("btn_preprocessor")
        self.horizontalLayout_3.addWidget(self.btn_preprocessor)
        self.btn_save_as = QtWidgets.QPushButton(self.widget)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_save_as.setIcon(icon6)
        self.btn_save_as.setObjectName("btn_save_as")
        self.horizontalLayout_3.addWidget(self.btn_save_as)
        self.btn_save_file = QtWidgets.QPushButton(self.widget)
        self.btn_save_file.setIcon(icon6)
        self.btn_save_file.setObjectName("btn_save_file")
        self.horizontalLayout_3.addWidget(self.btn_save_file)
        self.verticalLayout_9.addWidget(self.widget)
        self.widget_2 = QtWidgets.QWidget(self.tab_2)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.groupBox_5 = QtWidgets.QGroupBox(self.widget_2)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setObjectName("formLayout_4")
        self.afficherLesAxesLabel = QtWidgets.QLabel(self.groupBox_5)
        self.afficherLesAxesLabel.setObjectName("afficherLesAxesLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.afficherLesAxesLabel)
        self.display_axis = QtWidgets.QCheckBox(self.groupBox_5)
        self.display_axis.setObjectName("display_axis")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.display_axis)
        self.afficherLesTapesLabel = QtWidgets.QLabel(self.groupBox_5)
        self.afficherLesTapesLabel.setObjectName("afficherLesTapesLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.afficherLesTapesLabel)
        self.display_draw_steps = QtWidgets.QCheckBox(self.groupBox_5)
        self.display_draw_steps.setObjectName("display_draw_steps")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.display_draw_steps)
        self.afficherLaBoTeDeDLimitationLabel = QtWidgets.QLabel(self.groupBox_5)
        self.afficherLaBoTeDeDLimitationLabel.setObjectName("afficherLaBoTeDeDLimitationLabel")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.afficherLaBoTeDeDLimitationLabel)
        self.display_bounding_box = QtWidgets.QCheckBox(self.groupBox_5)
        self.display_bounding_box.setObjectName("display_bounding_box")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.display_bounding_box)
        self.inverserLAxeXLabel = QtWidgets.QLabel(self.groupBox_5)
        self.inverserLAxeXLabel.setObjectName("inverserLAxeXLabel")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeXLabel)
        self.reverse_display_x = QtWidgets.QCheckBox(self.groupBox_5)
        self.reverse_display_x.setObjectName("reverse_display_x")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.reverse_display_x)
        self.inverserLAxeYLabel = QtWidgets.QLabel(self.groupBox_5)
        self.inverserLAxeYLabel.setObjectName("inverserLAxeYLabel")
        self.formLayout_4.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeYLabel)
        self.reverse_display_y = QtWidgets.QCheckBox(self.groupBox_5)
        self.reverse_display_y.setChecked(True)
        self.reverse_display_y.setObjectName("reverse_display_y")
        self.formLayout_4.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.reverse_display_y)
        self.inverserLAxeZLabel = QtWidgets.QLabel(self.groupBox_5)
        self.inverserLAxeZLabel.setObjectName("inverserLAxeZLabel")
        self.formLayout_4.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeZLabel)
        self.reverse_display_z = QtWidgets.QCheckBox(self.groupBox_5)
        self.reverse_display_z.setObjectName("reverse_display_z")
        self.formLayout_4.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.reverse_display_z)
        self.verticalLayout.addLayout(self.formLayout_4)
        self.line = QtWidgets.QFrame(self.groupBox_5)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.formLayout_5 = QtWidgets.QFormLayout()
        self.formLayout_5.setObjectName("formLayout_5")
        self.label_5 = QtWidgets.QLabel(self.groupBox_5)
        self.label_5.setObjectName("label_5")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.space_x = QtWidgets.QLCDNumber(self.groupBox_5)
        self.space_x.setStyleSheet("color:DarkOrange;")
        self.space_x.setSegmentStyle(QtWidgets.QLCDNumber.Flat)
        self.space_x.setProperty("value", 0.0)
        self.space_x.setObjectName("space_x")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.space_x)
        self.label_11 = QtWidgets.QLabel(self.groupBox_5)
        self.label_11.setObjectName("label_11")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_11)
        self.space_y = QtWidgets.QLCDNumber(self.groupBox_5)
        self.space_y.setStyleSheet("color:DarkOrange;")
        self.space_y.setLineWidth(1)
        self.space_y.setSegmentStyle(QtWidgets.QLCDNumber.Flat)
        self.space_y.setObjectName("space_y")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.space_y)
        self.label_12 = QtWidgets.QLabel(self.groupBox_5)
        self.label_12.setObjectName("label_12")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_12)
        self.space_z = QtWidgets.QLCDNumber(self.groupBox_5)
        self.space_z.setStyleSheet("color:DarkOrange;")
        self.space_z.setSmallDecimalPoint(False)
        self.space_z.setSegmentStyle(QtWidgets.QLCDNumber.Flat)
        self.space_z.setObjectName("space_z")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.space_z)
        self.verticalLayout.addLayout(self.formLayout_5)
        self.horizontalLayout_5.addWidget(self.groupBox_5)
        self.fileview = QtWidgets.QGraphicsView(self.widget_2)
        self.fileview.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.fileview.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)
        self.fileview.setObjectName("fileview")
        self.horizontalLayout_5.addWidget(self.fileview)
        self.code_edit = QtWidgets.QTextEdit(self.widget_2)
        self.code_edit.setObjectName("code_edit")
        self.horizontalLayout_5.addWidget(self.code_edit)
        self.verticalLayout_9.addWidget(self.widget_2)
        self.tabWidget_2.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.tab_3)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.serial_monitor = QtWidgets.QTextEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.serial_monitor.sizePolicy().hasHeightForWidth())
        self.serial_monitor.setSizePolicy(sizePolicy)
        self.serial_monitor.setMinimumSize(QtCore.QSize(0, 300))
        self.serial_monitor.setMaximumSize(QtCore.QSize(16777215, 2000))
        self.serial_monitor.setStyleSheet("background-color:black;")
        self.serial_monitor.setReadOnly(True)
        self.serial_monitor.setObjectName("serial_monitor")
        self.verticalLayout_2.addWidget(self.serial_monitor)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.command_edit = QtWidgets.QLineEdit(self.groupBox)
        self.command_edit.setObjectName("command_edit")
        self.horizontalLayout_4.addWidget(self.command_edit)
        self.btn_command = QtWidgets.QPushButton(self.groupBox)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/run.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_command.setIcon(icon7)
        self.btn_command.setObjectName("btn_command")
        self.horizontalLayout_4.addWidget(self.btn_command)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.btn_send_current_file = QtWidgets.QPushButton(self.groupBox)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/upload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_send_current_file.setIcon(icon8)
        self.btn_send_current_file.setObjectName("btn_send_current_file")
        self.verticalLayout_2.addWidget(self.btn_send_current_file)
        self.sending_progress = QtWidgets.QProgressBar(self.groupBox)
        self.sending_progress.setProperty("value", 0)
        self.sending_progress.setObjectName("sending_progress")
        self.verticalLayout_2.addWidget(self.sending_progress)
        self.gridLayout_3.addWidget(self.groupBox, 1, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(self.tab_3)
        self.tabWidget.setEnabled(True)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_command = QtWidgets.QWidget()
        self.tab_command.setObjectName("tab_command")
        self.gridLayout = QtWidgets.QGridLayout(self.tab_command)
        self.gridLayout.setObjectName("gridLayout")
        self.grp_cmd = QtWidgets.QGroupBox(self.tab_command)
        self.grp_cmd.setObjectName("grp_cmd")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.grp_cmd)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btn_x_plus = QtWidgets.QPushButton(self.grp_cmd)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/right.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_x_plus.setIcon(icon9)
        self.btn_x_plus.setObjectName("btn_x_plus")
        self.gridLayout_2.addWidget(self.btn_x_plus, 1, 2, 1, 1)
        self.btn_y_plus = QtWidgets.QPushButton(self.grp_cmd)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/up.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_y_plus.setIcon(icon10)
        self.btn_y_plus.setObjectName("btn_y_plus")
        self.gridLayout_2.addWidget(self.btn_y_plus, 0, 1, 1, 1)
        self.btn_z_plus = QtWidgets.QPushButton(self.grp_cmd)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/up.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.btn_z_plus.setIcon(icon11)
        self.btn_z_plus.setObjectName("btn_z_plus")
        self.gridLayout_2.addWidget(self.btn_z_plus, 0, 3, 1, 1)
        self.btn_z_minus = QtWidgets.QPushButton(self.grp_cmd)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/down.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.btn_z_minus.setIcon(icon12)
        self.btn_z_minus.setObjectName("btn_z_minus")
        self.gridLayout_2.addWidget(self.btn_z_minus, 2, 3, 1, 1)
        self.btn_y_minus = QtWidgets.QPushButton(self.grp_cmd)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/down.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_y_minus.setIcon(icon13)
        self.btn_y_minus.setObjectName("btn_y_minus")
        self.gridLayout_2.addWidget(self.btn_y_minus, 2, 1, 1, 1)
        self.btn_x_minus = QtWidgets.QPushButton(self.grp_cmd)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/left.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_x_minus.setIcon(icon14)
        self.btn_x_minus.setObjectName("btn_x_minus")
        self.gridLayout_2.addWidget(self.btn_x_minus, 1, 0, 1, 1)
        self.btn_go_to_zero = QtWidgets.QPushButton(self.grp_cmd)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/origin.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_go_to_zero.setIcon(icon15)
        self.btn_go_to_zero.setIconSize(QtCore.QSize(64, 64))
        self.btn_go_to_zero.setFlat(False)
        self.btn_go_to_zero.setObjectName("btn_go_to_zero")
        self.gridLayout_2.addWidget(self.btn_go_to_zero, 1, 1, 1, 1)
        self.verticalLayout_6.addLayout(self.gridLayout_2)
        self.gridLayout.addWidget(self.grp_cmd, 1, 0, 1, 1)
        self.grp_auto = QtWidgets.QGroupBox(self.tab_command)
        self.grp_auto.setObjectName("grp_auto")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.grp_auto)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.label_6 = QtWidgets.QLabel(self.grp_auto)
        self.label_6.setWordWrap(True)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_14.addWidget(self.label_6)
        self.auto_cmd_1 = QtWidgets.QWidget(self.grp_auto)
        self.auto_cmd_1.setObjectName("auto_cmd_1")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.auto_cmd_1)
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_7 = QtWidgets.QLabel(self.auto_cmd_1)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_10.addWidget(self.label_7)
        self.auto_cmd_axis = QtWidgets.QComboBox(self.auto_cmd_1)
        self.auto_cmd_axis.setObjectName("auto_cmd_axis")
        self.auto_cmd_axis.addItem("")
        self.auto_cmd_axis.addItem("")
        self.auto_cmd_axis.addItem("")
        self.horizontalLayout_10.addWidget(self.auto_cmd_axis)
        self.label_8 = QtWidgets.QLabel(self.auto_cmd_1)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_10.addWidget(self.label_8)
        self.auto_cmd_number = QtWidgets.QSpinBox(self.auto_cmd_1)
        self.auto_cmd_number.setProperty("value", 1)
        self.auto_cmd_number.setObjectName("auto_cmd_number")
        self.horizontalLayout_10.addWidget(self.auto_cmd_number)
        self.auto_cmd_type = QtWidgets.QComboBox(self.auto_cmd_1)
        self.auto_cmd_type.setObjectName("auto_cmd_type")
        self.auto_cmd_type.addItem("")
        self.auto_cmd_type.addItem("")
        self.horizontalLayout_10.addWidget(self.auto_cmd_type)
        self.label_9 = QtWidgets.QLabel(self.auto_cmd_1)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_10.addWidget(self.label_9)
        self.auto_cmd_step = QtWidgets.QSpinBox(self.auto_cmd_1)
        self.auto_cmd_step.setMinimum(-1000)
        self.auto_cmd_step.setMaximum(1000)
        self.auto_cmd_step.setObjectName("auto_cmd_step")
        self.horizontalLayout_10.addWidget(self.auto_cmd_step)
        self.label_10 = QtWidgets.QLabel(self.auto_cmd_1)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_10.addWidget(self.label_10)
        self.verticalLayout_14.addWidget(self.auto_cmd_1)
        self.auto_cmd_2 = QtWidgets.QWidget(self.grp_auto)
        self.auto_cmd_2.setObjectName("auto_cmd_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.auto_cmd_2)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_13 = QtWidgets.QLabel(self.auto_cmd_2)
        self.label_13.setTextFormat(QtCore.Qt.PlainText)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_6.addWidget(self.label_13)
        self.auto_cmd_axis_2 = QtWidgets.QComboBox(self.auto_cmd_2)
        self.auto_cmd_axis_2.setObjectName("auto_cmd_axis_2")
        self.auto_cmd_axis_2.addItem("")
        self.auto_cmd_axis_2.addItem("")
        self.auto_cmd_axis_2.addItem("")
        self.horizontalLayout_6.addWidget(self.auto_cmd_axis_2)
        self.label_14 = QtWidgets.QLabel(self.auto_cmd_2)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_6.addWidget(self.label_14)
        self.auto_cmd_step_2 = QtWidgets.QSpinBox(self.auto_cmd_2)
        self.auto_cmd_step_2.setMinimum(-1000)
        self.auto_cmd_step_2.setMaximum(1000)
        self.auto_cmd_step_2.setObjectName("auto_cmd_step_2")
        self.horizontalLayout_6.addWidget(self.auto_cmd_step_2)
        self.label_15 = QtWidgets.QLabel(self.auto_cmd_2)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_6.addWidget(self.label_15)
        self.verticalLayout_14.addWidget(self.auto_cmd_2)
        self.btn_run_auto_cmd = QtWidgets.QPushButton(self.grp_auto)
        self.btn_run_auto_cmd.setIcon(icon7)
        self.btn_run_auto_cmd.setObjectName("btn_run_auto_cmd")
        self.verticalLayout_14.addWidget(self.btn_run_auto_cmd)
        self.gridLayout.addWidget(self.grp_auto, 2, 0, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(self.tab_command)
        self.groupBox_6.setObjectName("groupBox_6")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_6)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_16 = QtWidgets.QLabel(self.groupBox_6)
        self.label_16.setWordWrap(True)
        self.label_16.setObjectName("label_16")
        self.verticalLayout_5.addWidget(self.label_16)
        self.widget_4 = QtWidgets.QWidget(self.groupBox_6)
        self.widget_4.setObjectName("widget_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.widget_4)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_17 = QtWidgets.QLabel(self.widget_4)
        self.label_17.setObjectName("label_17")
        self.horizontalLayout_7.addWidget(self.label_17)
        self.custom_cmd_number = QtWidgets.QSpinBox(self.widget_4)
        self.custom_cmd_number.setMinimum(1)
        self.custom_cmd_number.setMaximum(1000)
        self.custom_cmd_number.setObjectName("custom_cmd_number")
        self.horizontalLayout_7.addWidget(self.custom_cmd_number)
        self.label_18 = QtWidgets.QLabel(self.widget_4)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_7.addWidget(self.label_18)
        self.verticalLayout_5.addWidget(self.widget_4)
        self.custom_cmd = QtWidgets.QTextEdit(self.groupBox_6)
        self.custom_cmd.setObjectName("custom_cmd")
        self.verticalLayout_5.addWidget(self.custom_cmd)
        self.btn_run_custom_cmd = QtWidgets.QPushButton(self.groupBox_6)
        self.btn_run_custom_cmd.setIcon(icon7)
        self.btn_run_custom_cmd.setObjectName("btn_run_custom_cmd")
        self.verticalLayout_5.addWidget(self.btn_run_custom_cmd)
        self.gridLayout.addWidget(self.groupBox_6, 1, 1, 2, 1)
        self.btn_set_origin = QtWidgets.QPushButton(self.tab_command)
        self.btn_set_origin.setIcon(icon15)
        self.btn_set_origin.setObjectName("btn_set_origin")
        self.gridLayout.addWidget(self.btn_set_origin, 0, 0, 1, 2)
        self.tabWidget.addTab(self.tab_command, "")
        self.tab_config = QtWidgets.QWidget()
        self.tab_config.setObjectName("tab_config")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.tab_config)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_3 = QtWidgets.QLabel(self.tab_config)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_9.addWidget(self.label_3)
        self.config_list = QtWidgets.QComboBox(self.tab_config)
        self.config_list.setObjectName("config_list")
        self.config_list.addItem("")
        self.horizontalLayout_9.addWidget(self.config_list)
        self.verticalLayout_11.addLayout(self.horizontalLayout_9)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_config)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setObjectName("groupBox_3")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox_3)
        self.formLayout_2.setObjectName("formLayout_2")
        self.modeDePilotageLabel_2 = QtWidgets.QLabel(self.groupBox_3)
        self.modeDePilotageLabel_2.setObjectName("modeDePilotageLabel_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.modeDePilotageLabel_2)
        self.drive_y = QtWidgets.QComboBox(self.groupBox_3)
        self.drive_y.setObjectName("drive_y")
        self.drive_y.addItem("")
        self.drive_y.addItem("")
        self.drive_y.addItem("")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.drive_y)
        self.ratioLabel_2 = QtWidgets.QLabel(self.groupBox_3)
        self.ratioLabel_2.setObjectName("ratioLabel_2")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.ratioLabel_2)
        self.ratio_y = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.ratio_y.setObjectName("ratio_y")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.ratio_y)
        self.jeuLabel_2 = QtWidgets.QLabel(self.groupBox_3)
        self.jeuLabel_2.setObjectName("jeuLabel_2")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.jeuLabel_2)
        self.play_y = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.play_y.setObjectName("play_y")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.play_y)
        self.inverserLAxeLabel_2 = QtWidgets.QLabel(self.groupBox_3)
        self.inverserLAxeLabel_2.setObjectName("inverserLAxeLabel_2")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeLabel_2)
        self.reverse_y = QtWidgets.QCheckBox(self.groupBox_3)
        self.reverse_y.setObjectName("reverse_y")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.reverse_y)
        self.gridLayout_4.addWidget(self.groupBox_3, 0, 1, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_config)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.formLayout = QtWidgets.QFormLayout(self.groupBox_2)
        self.formLayout.setObjectName("formLayout")
        self.modeDePilotageLabel = QtWidgets.QLabel(self.groupBox_2)
        self.modeDePilotageLabel.setObjectName("modeDePilotageLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.modeDePilotageLabel)
        self.drive_x = QtWidgets.QComboBox(self.groupBox_2)
        self.drive_x.setObjectName("drive_x")
        self.drive_x.addItem("")
        self.drive_x.addItem("")
        self.drive_x.addItem("")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.drive_x)
        self.ratioLabel = QtWidgets.QLabel(self.groupBox_2)
        self.ratioLabel.setObjectName("ratioLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.ratioLabel)
        self.ratio_x = QtWidgets.QDoubleSpinBox(self.groupBox_2)
        self.ratio_x.setObjectName("ratio_x")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.ratio_x)
        self.jeuLabel = QtWidgets.QLabel(self.groupBox_2)
        self.jeuLabel.setObjectName("jeuLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.jeuLabel)
        self.play_x = QtWidgets.QDoubleSpinBox(self.groupBox_2)
        self.play_x.setObjectName("play_x")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.play_x)
        self.inverserLAxeLabel = QtWidgets.QLabel(self.groupBox_2)
        self.inverserLAxeLabel.setObjectName("inverserLAxeLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeLabel)
        self.reverse_x = QtWidgets.QCheckBox(self.groupBox_2)
        self.reverse_x.setObjectName("reverse_x")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.reverse_x)
        self.gridLayout_4.addWidget(self.groupBox_2, 0, 0, 1, 1)
        self.verticalLayout_12 = QtWidgets.QVBoxLayout()
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.btn_send_config = QtWidgets.QPushButton(self.tab_config)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap("../sivi_cnc_driver/rc/connect.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_send_config.setIcon(icon16)
        self.btn_send_config.setObjectName("btn_send_config")
        self.verticalLayout_12.addWidget(self.btn_send_config)
        self.btn_save_config = QtWidgets.QPushButton(self.tab_config)
        self.btn_save_config.setIcon(icon6)
        self.btn_save_config.setObjectName("btn_save_config")
        self.verticalLayout_12.addWidget(self.btn_save_config)
        self.btn_save_config_as = QtWidgets.QPushButton(self.tab_config)
        self.btn_save_config_as.setIcon(icon6)
        self.btn_save_config_as.setObjectName("btn_save_config_as")
        self.verticalLayout_12.addWidget(self.btn_save_config_as)
        self.gridLayout_4.addLayout(self.verticalLayout_12, 1, 1, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab_config)
        self.groupBox_4.setObjectName("groupBox_4")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBox_4)
        self.formLayout_3.setObjectName("formLayout_3")
        self.modeDePilotageLabel_3 = QtWidgets.QLabel(self.groupBox_4)
        self.modeDePilotageLabel_3.setObjectName("modeDePilotageLabel_3")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.modeDePilotageLabel_3)
        self.drive_z = QtWidgets.QComboBox(self.groupBox_4)
        self.drive_z.setObjectName("drive_z")
        self.drive_z.addItem("")
        self.drive_z.addItem("")
        self.drive_z.addItem("")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.drive_z)
        self.ratioLabel_3 = QtWidgets.QLabel(self.groupBox_4)
        self.ratioLabel_3.setObjectName("ratioLabel_3")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.ratioLabel_3)
        self.ratio_z = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        self.ratio_z.setObjectName("ratio_z")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.ratio_z)
        self.jeuLabel_3 = QtWidgets.QLabel(self.groupBox_4)
        self.jeuLabel_3.setObjectName("jeuLabel_3")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.jeuLabel_3)
        self.play_z = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        self.play_z.setObjectName("play_z")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.play_z)
        self.inverserLAxeLabel_3 = QtWidgets.QLabel(self.groupBox_4)
        self.inverserLAxeLabel_3.setObjectName("inverserLAxeLabel_3")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.inverserLAxeLabel_3)
        self.reverse_z = QtWidgets.QCheckBox(self.groupBox_4)
        self.reverse_z.setObjectName("reverse_z")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.reverse_z)
        self.gridLayout_4.addWidget(self.groupBox_4, 1, 0, 1, 1)
        self.verticalLayout_11.addLayout(self.gridLayout_4)
        self.tabWidget.addTab(self.tab_config, "")
        self.gridLayout_3.addWidget(self.tabWidget, 1, 1, 1, 1)
        self.groupBox_7 = QtWidgets.QGroupBox(self.tab_3)
        self.groupBox_7.setTitle("")
        self.groupBox_7.setObjectName("groupBox_7")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_7)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_19 = QtWidgets.QLabel(self.groupBox_7)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_2.addWidget(self.label_19)
        self.chk_fake_serial = QtWidgets.QCheckBox(self.groupBox_7)
        self.chk_fake_serial.setObjectName("chk_fake_serial")
        self.horizontalLayout_2.addWidget(self.chk_fake_serial)
        self.line_2 = QtWidgets.QFrame(self.groupBox_7)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_2.addWidget(self.line_2)
        self.btn_serial_ports_list = QtWidgets.QPushButton(self.groupBox_7)
        self.btn_serial_ports_list.setIcon(icon2)
        self.btn_serial_ports_list.setObjectName("btn_serial_ports_list")
        self.horizontalLayout_2.addWidget(self.btn_serial_ports_list)
        self.line_3 = QtWidgets.QFrame(self.groupBox_7)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_2.addWidget(self.line_3)
        self.label_20 = QtWidgets.QLabel(self.groupBox_7)
        self.label_20.setObjectName("label_20")
        self.horizontalLayout_2.addWidget(self.label_20)
        self.baudrate = QtWidgets.QComboBox(self.groupBox_7)
        self.baudrate.setObjectName("baudrate")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.baudrate.addItem("")
        self.horizontalLayout_2.addWidget(self.baudrate)
        self.label_21 = QtWidgets.QLabel(self.groupBox_7)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_2.addWidget(self.label_21)
        self.serial_ports_list = QtWidgets.QComboBox(self.groupBox_7)
        self.serial_ports_list.setObjectName("serial_ports_list")
        self.horizontalLayout_2.addWidget(self.serial_ports_list)
        self.btn_connect = QtWidgets.QPushButton(self.groupBox_7)
        self.btn_connect.setIcon(icon16)
        self.btn_connect.setObjectName("btn_connect")
        self.horizontalLayout_2.addWidget(self.btn_connect)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.gridLayout_3.addWidget(self.groupBox_7, 0, 0, 1, 2)
        self.tabWidget_2.addTab(self.tab_3, "")
        self.verticalLayout_4.addWidget(self.tabWidget_2)
        self.widget_5 = QtWidgets.QWidget(self.centralwidget)
        self.widget_5.setObjectName("widget_5")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.widget_5)
        self.horizontalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_2 = QtWidgets.QLabel(self.widget_5)
        self.label_2.setTextFormat(QtCore.Qt.RichText)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_8.addWidget(self.label_2)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.btn_license = QtWidgets.QPushButton(self.widget_5)
        self.btn_license.setObjectName("btn_license")
        self.horizontalLayout_8.addWidget(self.btn_license)
        self.btn_about_qt = QtWidgets.QPushButton(self.widget_5)
        self.btn_about_qt.setObjectName("btn_about_qt")
        self.horizontalLayout_8.addWidget(self.btn_about_qt)
        self.label_2.raise_()
        self.btn_license.raise_()
        self.btn_about_qt.raise_()
        self.verticalLayout_4.addWidget(self.widget_5)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        self.tabWidget_2.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(0)
        self.baudrate.setCurrentIndex(4)
        self.command_edit.returnPressed.connect(self.btn_command.click)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "SiviCNCDriver"))
        self.filename.setText(_translate("MainWindow", "Pas de fichier"))
        self.btn_file.setText(_translate("MainWindow", "Choisir le fichier"))
        self.btn_reload.setText(_translate("MainWindow", "Recharger"))
        self.btn_close.setText(_translate("MainWindow", "Fermer"))
        self.redraw.setText(_translate("MainWindow", "Redessiner"))
        self.btn_preprocessor.setText(_translate("MainWindow", "Utiliser le préprocesseur sur le fichier"))
        self.btn_save_as.setText(_translate("MainWindow", "Enregister sous..."))
        self.btn_save_file.setText(_translate("MainWindow", "Enregistrer"))
        self.groupBox_5.setTitle(_translate("MainWindow", "Affichage"))
        self.afficherLesAxesLabel.setText(_translate("MainWindow", "Afficher les axes"))
        self.afficherLesTapesLabel.setText(_translate("MainWindow", "Afficher les étapes"))
        self.afficherLaBoTeDeDLimitationLabel.setText(_translate("MainWindow", "Afficher la boîte de délimitation"))
        self.inverserLAxeXLabel.setText(_translate("MainWindow", "Retourner l\'axe X"))
        self.inverserLAxeYLabel.setText(_translate("MainWindow", "Retourner l\'axe Y"))
        self.inverserLAxeZLabel.setText(_translate("MainWindow", "Retourner l\'axe Z"))
        self.label_5.setText(_translate("MainWindow", "Développement selon X"))
        self.label_11.setText(_translate("MainWindow", "Développement selon Y"))
        self.label_12.setText(_translate("MainWindow", "Développement selon Z"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_2), _translate("MainWindow", "Fichier"))
        self.groupBox.setTitle(_translate("MainWindow", "Moniteur série"))
        self.serial_monitor.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'Ubuntu\'; font-size:11pt;\"><br /></p></body></html>"))
        self.label_4.setText(_translate("MainWindow", "Commande :"))
        self.btn_command.setText(_translate("MainWindow", "Envoyer"))
        self.btn_send_current_file.setText(_translate("MainWindow", "Envoyer le fichier courrant"))
        self.grp_cmd.setTitle(_translate("MainWindow", "Commande manuelle"))
        self.btn_x_plus.setText(_translate("MainWindow", "X+"))
        self.btn_y_plus.setText(_translate("MainWindow", "Y+"))
        self.btn_z_plus.setText(_translate("MainWindow", "Z+"))
        self.btn_z_minus.setText(_translate("MainWindow", "Z-"))
        self.btn_y_minus.setText(_translate("MainWindow", "Y-"))
        self.btn_x_minus.setText(_translate("MainWindow", "X-"))
        self.btn_go_to_zero.setText(_translate("MainWindow", "Aller à la position\n"
"Zéro"))
        self.grp_auto.setTitle(_translate("MainWindow", "Commandes automatiques"))
        self.label_6.setText(_translate("MainWindow", "Les commandes automatiques permettent d\'envoyer des commandes directement en pas afin de régler la machine."))
        self.label_7.setText(_translate("MainWindow", "L\'axe"))
        self.auto_cmd_axis.setItemText(0, _translate("MainWindow", "X"))
        self.auto_cmd_axis.setItemText(1, _translate("MainWindow", "Y"))
        self.auto_cmd_axis.setItemText(2, _translate("MainWindow", "Z"))
        self.label_8.setText(_translate("MainWindow", "effectue"))
        self.auto_cmd_type.setItemText(0, _translate("MainWindow", "aller"))
        self.auto_cmd_type.setItemText(1, _translate("MainWindow", "aller(s)-retour(s)"))
        self.label_9.setText(_translate("MainWindow", "de"))
        self.label_10.setText(_translate("MainWindow", "pas."))
        self.label_13.setText(_translate("MainWindow", "Entre chaque étape, l\'axe"))
        self.auto_cmd_axis_2.setItemText(0, _translate("MainWindow", "X"))
        self.auto_cmd_axis_2.setItemText(1, _translate("MainWindow", "Y"))
        self.auto_cmd_axis_2.setItemText(2, _translate("MainWindow", "Z"))
        self.label_14.setText(_translate("MainWindow", "se déplace de"))
        self.label_15.setText(_translate("MainWindow", "pas."))
        self.btn_run_auto_cmd.setText(_translate("MainWindow", "Envoyer"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Commandes personnalisées"))
        self.label_16.setText(_translate("MainWindow", "Les commandes personnalisées permettent d\'envoyer des commandes définies par l\'utilisateur un certain nombre de fois afin de régler la machine."))
        self.label_17.setText(_translate("MainWindow", "Répéter"))
        self.label_18.setText(_translate("MainWindow", "fois les commandes :"))
        self.btn_run_custom_cmd.setText(_translate("MainWindow", "Envoyer"))
        self.btn_set_origin.setText(_translate("MainWindow", "Définir l\'origine"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_command), _translate("MainWindow", "Commande"))
        self.label_3.setText(_translate("MainWindow", "Configuration :"))
        self.config_list.setItemText(0, _translate("MainWindow", "Nouvelle configuration"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Axe Y"))
        self.modeDePilotageLabel_2.setText(_translate("MainWindow", "Mode de pilotage"))
        self.drive_y.setItemText(0, _translate("MainWindow", "Normal"))
        self.drive_y.setItemText(1, _translate("MainWindow", "Couple maximal"))
        self.drive_y.setItemText(2, _translate("MainWindow", "Demi pas"))
        self.ratioLabel_2.setText(_translate("MainWindow", "Ratio"))
        self.ratio_y.setSuffix(_translate("MainWindow", " mm/pas"))
        self.jeuLabel_2.setText(_translate("MainWindow", "Jeu"))
        self.play_y.setSuffix(_translate("MainWindow", "  mm"))
        self.inverserLAxeLabel_2.setText(_translate("MainWindow", "Inverser l\'axe"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Axe X"))
        self.modeDePilotageLabel.setText(_translate("MainWindow", "Mode de pilotage"))
        self.drive_x.setItemText(0, _translate("MainWindow", "Normal"))
        self.drive_x.setItemText(1, _translate("MainWindow", "Couple maximal"))
        self.drive_x.setItemText(2, _translate("MainWindow", "Demi pas"))
        self.ratioLabel.setText(_translate("MainWindow", "Ratio"))
        self.ratio_x.setSuffix(_translate("MainWindow", " mm/pas"))
        self.jeuLabel.setText(_translate("MainWindow", "Jeu"))
        self.play_x.setSuffix(_translate("MainWindow", " mm"))
        self.inverserLAxeLabel.setText(_translate("MainWindow", "Inverser l\'axe"))
        self.btn_send_config.setText(_translate("MainWindow", "Envoyer la configuration"))
        self.btn_save_config.setText(_translate("MainWindow", "Enregister"))
        self.btn_save_config_as.setText(_translate("MainWindow", "Enregister sous"))
        self.groupBox_4.setTitle(_translate("MainWindow", "Axe Z"))
        self.modeDePilotageLabel_3.setText(_translate("MainWindow", "Mode de pilotage"))
        self.drive_z.setItemText(0, _translate("MainWindow", "Normal"))
        self.drive_z.setItemText(1, _translate("MainWindow", "Couple maximal"))
        self.drive_z.setItemText(2, _translate("MainWindow", "Demi pas"))
        self.ratioLabel_3.setText(_translate("MainWindow", "Ratio"))
        self.ratio_z.setSuffix(_translate("MainWindow", " mm/pas"))
        self.jeuLabel_3.setText(_translate("MainWindow", "Jeu"))
        self.play_z.setSuffix(_translate("MainWindow", " mm"))
        self.inverserLAxeLabel_3.setText(_translate("MainWindow", "Inverser l\'axe"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_config), _translate("MainWindow", "Configuration"))
        self.label_19.setText(_translate("MainWindow", "Émuler un port série "))
        self.btn_serial_ports_list.setText(_translate("MainWindow", "Lister les ports série"))
        self.label_20.setText(_translate("MainWindow", "Baudrate"))
        self.baudrate.setItemText(0, _translate("MainWindow", "300"))
        self.baudrate.setItemText(1, _translate("MainWindow", "1200"))
        self.baudrate.setItemText(2, _translate("MainWindow", "2400"))
        self.baudrate.setItemText(3, _translate("MainWindow", "4800"))
        self.baudrate.setItemText(4, _translate("MainWindow", "9600"))
        self.baudrate.setItemText(5, _translate("MainWindow", "19200"))
        self.baudrate.setItemText(6, _translate("MainWindow", "38400"))
        self.baudrate.setItemText(7, _translate("MainWindow", "57600"))
        self.baudrate.setItemText(8, _translate("MainWindow", "115200"))
        self.baudrate.setItemText(9, _translate("MainWindow", "230400"))
        self.baudrate.setItemText(10, _translate("MainWindow", "250000"))
        self.label_21.setText(_translate("MainWindow", "Port série"))
        self.btn_connect.setText(_translate("MainWindow", "Connecter"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_3), _translate("MainWindow", "Port Série"))
        self.label_2.setText(_translate("MainWindow", "<html><head/><body><p>SiviCNCDriver est un logiciel développé par klafyvel. Pour plus d\'informations voir <a href=\"http://sivigik.com\"><span style=\" text-decoration: underline; color:#4c6b8a;\">sivigik.com</span></a> .</p></body></html>"))
        self.btn_license.setText(_translate("MainWindow", "Licence"))
        self.btn_about_qt.setText(_translate("MainWindow", "À propos de Qt"))




"""
The interface module
====================

Provides the MainWindow class. 
"""

import serial
from math import atan2, sqrt, pi
import os
import json

from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5 import QtGui
from PyQt5.QtWidgets import *

from sivicncdriver import settings
from sivicncdriver.settings import logger
from sivicncdriver.gcode import parse
from sivicncdriver.serial_list import serial_ports
from sivicncdriver.serial_manager import SerialManager
from sivicncdriver.preprocessor import PreprocessorDialog
from sivicncdriver.arc_calculator import arc_to_segments

from .main_window import Ui_MainWindow

__all__ = ['MainWindow']


class SendFileThread(QThread):
    """
    A thread to send file without blocking the main thread.
    """
    update_progress = pyqtSignal(int)

    def __init__(self, serial_manager, gcode):
        """
        The __init__ method.

        :param serial_manager: The main window's serial manager
        :param gcode: The gcode which is to be sent
        :type serial_manager: SerialManager
        :type gcode: str
        """
        QThread.__init__(self)
        self.gcode = gcode
        self.serial_manager = serial_manager
        self.user_stop = False

    def __del__(self):
        self.wait()

    @pyqtSlot()
    def stop(self):
        """
        A simple slot to tell the thread to stop.
        """
        self.user_stop = True

    def run(self):
        """
        Runs the thread.

        The commands are sent using the serial manager. If an error occurs or if
        the thread is stopped by the user, then it quits.
        """
        for n, l in enumerate(self.gcode):
            self.serial_manager.sendMsg(l)
            self.update_progress.emit(n)
            if not self.serial_manager.waitForConfirm() or self.user_stop:
                break


class SendAutoCmdThread(QThread):
    """
    A thread to send auto commands without blocking the main thread.
    """
    update_progress = pyqtSignal(int)

    def __init__(self, serial_manager, axis, step, n, axis2=None, step2=None):
        """
        The __init__ method.

        :param serial_manager: The main window's serial manager
        :param axis: The axis on which the thread will operate
        :param step: The number of step of one movement
        :param n: The number of movements
        :param axis2: You can ask a second axis to move between two first axis
            moves.
        :param step2: The number of steps the second axis has to move.
        :type serial_manager: SerialManager
        :type axis: str
        :type step: int
        :type n: int
        :type axis2: str
        :type step2: int
        """
        QThread.__init__(self)
        self.axis = axis
        self.step = step
        self.n = n
        self.serial_manager = serial_manager
        self.user_stop = False
        self.axis2 = axis2
        self.step2 = step2

    def __del__(self):
        self.wait()

    @pyqtSlot()
    def stop(self):
        """
        A simple slot to tell the thread to stop.
        """
        self.user_stop = True

    def run(self):
        """
        Runs the thread.

        The commands are sent using the serial manager. If an error occurs or if
        the thread is stopped by the user, then it quits.
        """
        for i in range(self.n):
            self.update_progress.emit(i)
            if not self.serial_manager.step(self.axis, self.step) or self.user_stop:
                break
            if self.axis2 and self.step2:
                if not self.serial_manager.step(self.axis2, self.step2) or self.user_stop:
                    break
            if not self.serial_manager.step(self.axis, -self.step) or self.user_stop:
                break
            if self.axis2 and self.step2:
                if not self.serial_manager.step(self.axis2, self.step2) or self.user_stop:
                    break


class SendCustomCommandThread(QThread):
    """
    A thread to send custom commands without blocking the main thread.
    """
    update_progress = pyqtSignal(int)

    def __init__(self, serial_manager, gcode, n):
        """
        The __init__ method.

        :param serial_manager: The main window's serial manager
        :param gcode: The gcode which is to be sent
        :param n: The number of times the command will be sent
        :type serial_manager: SerialManager
        :type gcode: str
        :type n: int
        """
        QThread.__init__(self)
        self.gcode = gcode
        self.serial_manager = serial_manager
        self.n = n
        self.user_stop = False

    def __del__(self):
        self.wait()

    @pyqtSlot()
    def stop(self):
        """
        A simple slot to tell the thread to stop.
        """
        self.user_stop = True

    def run(self):
        """
        Runs the thread.

        The commands are sent using the serial manager. If an error occurs or if
        the thread is stopped by the user, then it quits.
        """
        length = len(self.gcode)
        for k in range(self.n):
            for n, l in enumerate(self.gcode):
                self.update_progress.emit(k*length + n)
                self.serial_manager.sendMsg(l)
                if not self.serial_manager.waitForConfirm() or self.user_stop:
                    break


class MainWindow(QMainWindow, Ui_MainWindow):
    """
    The main window of the application.
    """

    def __init__(self):
        """
        The __init__ method.

        It will set the UI up, list available serial ports, list available
        configurations, connect the UI and so on.
        """
        super(MainWindow, self).__init__()
        self.setupUi(self)
        self.zoom = 1
        self.list_serials()
        self.list_configs()

        self.set_serial_mode("manual")

        self.serial_manager = SerialManager(serial.Serial(timeout=0))

        s = """Welcome to SiviCNCDriver, by Klafyvel from sivigik.com"""
        self.print(s, msg_type="info")

        self.sc = QGraphicsScene(self.fileview)
        self.attach_icons()
        self.connectUi()
        self.update_config(self.config_list.currentIndex())

        self.file_loaded = False

    def attach_icons(self):
        """
        Attach the icons to the buttons.

        Actually there must be a better way to do it (using the .ui file) but
        I could not figure out how.
        """        
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "siviIcon.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "load.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_file.setIcon(icon1)

        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "reload.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_reload.setIcon(icon2)

        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "writing.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.redraw.setIcon(icon3)

        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "work.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_preprocessor.setIcon(icon4)

        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "save.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_save_as.setIcon(icon5)
        self.btn_save_file.setIcon(icon5)

        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "run.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_command.setIcon(icon6)

        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "upload.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_send_current_file.setIcon(icon7)

        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "connect.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_connect.setIcon(icon8)

        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "origin.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_set_origin.setIcon(icon9)

        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "right.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_x_plus.setIcon(icon10)

        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "up.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_y_plus.setIcon(icon11)

        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "down.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_y_minus.setIcon(icon12)

        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "left.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_x_minus.setIcon(icon13)

        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "up.png")), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.btn_z_plus.setIcon(icon14)

        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR, "down.png")), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.btn_z_minus.setIcon(icon15)

        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(os.path.join(
            settings.RC_DIR,"close.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_close.setIcon(icon16)

        self.btn_go_to_zero.setIcon(icon9)
        self.btn_run_auto_cmd.setIcon(icon6)
        self.btn_send_config.setIcon(icon8)
        self.btn_save_config.setIcon(icon5)
        self.btn_save_config_as.setIcon(icon5)
        self.btn_run_custom_cmd.setIcon(icon6)
        self.btn_serial_ports_list.setIcon(icon2)

    def connectUi(self):
        """
        Connects The UI signals and slots.
        """
        logger.debug("Connecting Ui.")
        self.btn_serial_ports_list.clicked.connect(self.list_serials)

        self.btn_y_plus.pressed.connect(
            self.serial_manager.start_continuous_y_forward)
        self.btn_y_minus.pressed.connect(
            self.serial_manager.start_continuous_y_backward)
        self.btn_x_plus.pressed.connect(
            self.serial_manager.start_continuous_x_forward)
        self.btn_x_minus.pressed.connect(
            self.serial_manager.start_continuous_x_backward)
        self.btn_z_plus.pressed.connect(
            self.serial_manager.start_continuous_z_forward)
        self.btn_z_minus.pressed.connect(
            self.serial_manager.start_continuous_z_backward)

        self.btn_y_plus.released.connect(self.serial_manager.stop_continuous_y)
        self.btn_y_minus.released.connect(
            self.serial_manager.stop_continuous_y)
        self.btn_x_plus.released.connect(self.serial_manager.stop_continuous_x)
        self.btn_x_minus.released.connect(
            self.serial_manager.stop_continuous_x)
        self.btn_z_plus.released.connect(self.serial_manager.stop_continuous_z)
        self.btn_z_minus.released.connect(
            self.serial_manager.stop_continuous_z)

        self.btn_set_origin.clicked.connect(self.serial_manager.set_origin)
        self.btn_go_to_zero.clicked.connect(self.serial_manager.goto_origin)

        self.auto_cmd_type.currentIndexChanged.connect(
            self.manage_auto_cmd_number)
        self.btn_run_auto_cmd.clicked.connect(self.auto_cmd)
        self.manage_auto_cmd_number(self.auto_cmd_type.currentIndex())

        self.btn_run_custom_cmd.clicked.connect(self.run_custom_cmd)

        self.chk_fake_serial.stateChanged.connect(
            self.manage_emulate_serial_port)
        self.btn_send_current_file.clicked.connect(self.send_file)
        self.btn_command.clicked.connect(self.send_cmd)

        self.config_list.currentIndexChanged.connect(self.update_config)
        self.btn_save_config.clicked.connect(self.save_config)
        self.btn_save_config_as.clicked.connect(self.save_config_as)
        self.btn_send_config.clicked.connect(self.send_config)

        self.btn_connect.clicked.connect(self.manage_connection)

        self.btn_file.clicked.connect(self.choose_file)
        self.btn_reload.clicked.connect(self.load_file)
        self.btn_save_file.clicked.connect(self.save_file)
        self.btn_save_as.clicked.connect(self.save_file_as)
        self.redraw.clicked.connect(self.draw_file)
        self.btn_close.clicked.connect(self.close_file)

        self.serial_manager.send_print.connect(self.print)

        self.btn_preprocessor.clicked.connect(self.run_preprocessor)

        self.display_axis.clicked.connect(self.draw_file)
        self.display_draw_steps.clicked.connect(self.draw_file)
        self.display_bounding_box.clicked.connect(self.draw_file)
        self.reverse_display_x.clicked.connect(self.draw_file)
        self.reverse_display_y.clicked.connect(self.draw_file)
        self.reverse_display_z.clicked.connect(self.draw_file)

        self.btn_license.clicked.connect(self.about_license)
        self.btn_about_qt.clicked.connect(self.about_qt)

        self.zoomIn_shortcut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_Plus), self)
        self.zoomOut_shortcut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_Minus), self)

        self.zoomIn_shortcut.activated.connect(self.zoomIn)
        self.zoomOut_shortcut.activated.connect(self.zoomOut)

        self.sc.selectionChanged.connect(self.highlight_selected_path)

    def list_serials(self):
        """
        Lists available serials ports.
        """
        logger.debug("Listing available serial ports.")
        l = serial_ports()
        for i in range(self.serial_ports_list.count()):
            self.serial_ports_list.removeItem(0)
        for i in l:
            logger.info("Found {}".format(i))
            self.serial_ports_list.addItem(i)

    def list_configs(self):
        """
        Lists available configurations.
        """
        for _ in range(self.config_list.count()):
            self.config_list.removeItem(0)
        logger.debug("Listing available configurations.")
        config_dir = os.path.join(settings.APP_DIR, "configs", "")
        for f in os.listdir(config_dir):
            if f.endswith(".json"):
                logger.debug("Found {}".format(f))
                self.config_list.addItem(f[:-5])
        self.config_list.addItem("Nouvelle configuration")

    def set_serial_mode(self, mode):
        """
        Change serial mode.

        :param mode: can be "manual" or "file"
        :type mode: str
        """
        if mode == "manual":
            logger.debug("Setting manual mode.")
            self.btn_set_origin.setEnabled(True)
            self.grp_cmd.setEnabled(True)
            self.grp_auto.setEnabled(True)
            # self.btn_y_plus.pressed.connect()
        elif mode == "file":
            logger.debug("Setting file mode.")
            self.btn_set_origin.setEnabled(False)
            self.grp_cmd.setEnabled(False)
            self.grp_auto.setEnabled(False)

    def reset_config(self):
        """
        Resets the configuration.
        """
        self.drive_x.setCurrentIndex(0)
        self.drive_y.setCurrentIndex(0)
        self.drive_z.setCurrentIndex(0)

        self.ratio_x.setValue(1)
        self.ratio_y.setValue(1)
        self.ratio_z.setValue(1)

        self.play_x.setValue(0)
        self.play_y.setValue(0)
        self.play_z.setValue(0)

        self.reverse_x.setChecked(False)
        self.reverse_y.setChecked(False)
        self.reverse_z.setChecked(False)

        logger.info("Reset config.")

    def config_as_dict(self):
        """
        Get the configuration as a dict.

        :return: The configuration as a dict.
        :rtype: dict
        """
        return {
            "x_drive": self.drive_x.currentIndex(),
            "x_ratio": self.ratio_x.value(),
            "x_play": self.play_x.value(),
            "x_reverse": bool(self.reverse_x.checkState()),
            "y_drive": self.drive_y.currentIndex(),
            "y_ratio": self.ratio_y.value(),
            "y_play": self.play_y.value(),
            "y_reverse": bool(self.reverse_y.checkState()),
            "z_drive": self.drive_z.currentIndex(),
            "z_ratio": self.ratio_z.value(),
            "z_play": self.play_z.value(),
            "z_reverse": bool(self.reverse_z.checkState()),
        }

    # Slots
    @pyqtSlot(str, str)
    def print(self, txt, msg_type="operator"):
        """
        Prints a message on the application console.

        :param txt: The message
        :param msg_type: The type of the message. Can be "operator", "machine",
            "error" or "info"
        :type txt: str
        :type msg_type: str
        """

        msg = "{}"
        if msg_type == "operator":
            msg = "\n<br /><span style='color:orange;'> <strong>&gt;&gt;&gt;</strong> {}</span>"
        elif msg_type == "machine":
            msg = "\n<br /><span style='color:yellow;'>{}</span>"
        elif msg_type == "error":
            msg = "\n<br /><span style='color:red;'><strong>{}</strong></span>"
        elif msg_type == "info":
            msg = "\n<br /><span style='color:DarkOrange;'><strong>{}</strong></span>"
        self.serial_monitor.moveCursor(QTextCursor.End)
        self.serial_monitor.insertHtml(msg.format(txt))
        self.serial_monitor.moveCursor(QTextCursor.End)

    @pyqtSlot(int)
    def manage_auto_cmd_number(self, n):
        """
        Enable the widgets for auto commands
        """
        self.auto_cmd_number.setValue(1)
        self.auto_cmd_number.setEnabled(n != 0)
        self.auto_cmd_2.setEnabled(n != 0)

    @pyqtSlot()
    def auto_cmd(self):
        """
        Sends auto commands using a thread if they are too long.
        """
        logger.info("Sending auto command.")
        self.print("Sending auto command.", "info")

        axis = self.auto_cmd_axis.currentText()
        n = self.auto_cmd_number.value()
        step = self.auto_cmd_step.value()

        if self.auto_cmd_type.currentIndex() == 0:
            self.serial_manager.step(axis, step)
            self.print("Done.", "info")
            logger.info("Auto command sent.")
        else:
            axis2 = self.auto_cmd_axis_2.currentText()
            step2 = self.auto_cmd_step_2.value()
            self.send_thread = SendAutoCmdThread(
                self.serial_manager, axis, step, n, axis2, step2)

            self.sending_progress.setMaximum(n)
            self.sending_progress.setValue(0)
            self.btn_send_current_file.setText("Annuler l'envoi")
            self.btn_send_current_file.clicked.disconnect()
            self.btn_send_current_file.clicked.connect(self.send_thread.stop)
            self.tabWidget.setEnabled(False)

            self.send_thread.finished.connect(self.file_sent)
            self.send_thread.update_progress.connect(self.update_progress)
            self.send_thread.start()

    @pyqtSlot()
    def run_custom_cmd(self):
        """
        Sends a custom command using a thread.
        """
        logger.info("Sending custom command.")
        self.print("Sending custom command.", "info")
        gcode = self.custom_cmd.toPlainText().split('\n')
        n = self.custom_cmd_number.value()

        self.send_thread = SendCustomCommandThread(self.serial_manager, gcode, n)
        self.sending_progress.setMaximum(n*len(gcode))
        self.sending_progress.setValue(0)
        self.btn_send_current_file.setText("Annuler l'envoi")
        self.btn_send_current_file.clicked.disconnect()
        self.btn_send_current_file.clicked.connect(self.send_thread.stop)
        self.tabWidget.setEnabled(False)

        self.send_thread.finished.connect(self.file_sent)
        self.send_thread.update_progress.connect(self.update_progress)
        self.send_thread.start()

    @pyqtSlot(int)
    def manage_emulate_serial_port(self, s):
        """
        Enable widgets for serial port emulation.
        """
        st = bool(s)
        self.serial_manager.fake_mode = st
        self.baudrate.setEnabled(not st)
        self.serial_ports_list.setEnabled(not st)
        self.btn_serial_ports_list.setEnabled(not st)
        self.btn_connect.setEnabled(not st)
        if st:
            self.print("Emulating serial port.", "info")
        else:
            self.print("Exiting serial port emulation.", "info")

    @pyqtSlot()
    def send_file(self):
        """
        Send a file using a different thread.
        """
        logger.info("Sending file.")
        self.print("Sending file.", "info")
        gcode = self.code_edit.toPlainText().split('\n')
        self.send_thread = SendFileThread(self.serial_manager, gcode)

        self.sending_progress.setMaximum(len(gcode))
        self.sending_progress.setValue(0)
        self.btn_send_current_file.setText("Annuler l'envoi")
        self.btn_send_current_file.clicked.disconnect()
        self.btn_send_current_file.clicked.connect(self.send_thread.stop)
        self.tabWidget.setEnabled(False)

        self.send_thread.finished.connect(self.auto_cmd_sent)
        self.send_thread.update_progress.connect(self.update_progress)
        self.send_thread.start()

    @pyqtSlot(int)
    def update_progress(self, s):
        """
        Updates the progress bar.
        """
        self.sending_progress.setValue(s)

    @pyqtSlot()
    def file_sent(self):
        """
        Manages the end of a file sending.
        """
        if self.send_thread.user_stop:
            self.print("Stopped by user.", "error")
            logger.error("File sending stopped by user.")
        else:
            self.print("Done.", "info")
            logger.info("File sent.")
        self.sending_progress.setValue(0)
        self.btn_send_current_file.setText("Envoyer le fichier courrant")
        self.btn_send_current_file.clicked.disconnect()
        self.btn_send_current_file.clicked.connect(self.send_file)
        self.tabWidget.setEnabled(True)

    @pyqtSlot()
    def auto_cmd_sent(self):
        """
        Manages the end of an auto command sending.
        """
        if self.send_thread.user_stop:
            self.print("Stopped by user.", "error")
            logger.error("Auto command sending stopped by user.")
        else:
            self.print("Done.", "info")
            logger.info("Auto command sent.")
        self.sending_progress.setValue(0)
        self.btn_send_current_file.setText("Envoyer le fichier courrant")
        self.btn_send_current_file.clicked.disconnect()
        self.btn_send_current_file.clicked.connect(self.send_file)
        self.tabWidget.setEnabled(True)

    @pyqtSlot()
    def send_cmd(self):
        """
        Sends an user command.
        """
        gcode = self.command_edit.text()
        self.serial_manager.sendMsg(gcode)
        self.serial_manager.waitForConfirm()

    @pyqtSlot(int)
    def update_config(self, i):
        """
        Updates the configuration widgets.
        """
        nb_config = self.config_list.count()
        if i == nb_config-1:
            self.reset_config()
        else:
            file = self.config_list.currentText() + ".json"
            file = os.path.join(settings.APP_DIR, "configs", file)
            logger.info("Loading config {}".format(file))
            config = {}
            with open(file) as f:
                config = json.load(f)
            self.drive_x.setCurrentIndex(config.get("x_drive", 0))
            self.drive_y.setCurrentIndex(config.get("y_drive", 0))
            self.drive_z.setCurrentIndex(config.get("z_drive", 0))

            self.ratio_x.setValue(config.get("x_ratio", 1))
            self.ratio_y.setValue(config.get("y_ratio", 1))
            self.ratio_z.setValue(config.get("z_ratio", 1))

            self.play_x.setValue(config.get("x_play", 0))
            self.play_y.setValue(config.get("y_play", 0))
            self.play_z.setValue(config.get("z_play", 0))

            self.reverse_x.setChecked(config.get("x_reverse", False))
            self.reverse_y.setChecked(config.get("y_reverse", False))
            self.reverse_z.setChecked(config.get("z_reverse", False))

    @pyqtSlot()
    def save_config(self, filename=None):
        """
        Saves a configuration.
        :param filename: The name of the file.
        :type filename: str
        """
        logger.info("Saving configuration.")
        logger.debug("Filename given : {}".format(filename))
        current_config = self.config_list.currentIndex()
        nb_config = self.config_list.count()
        if current_config == nb_config-1 and not filename:
            self.save_config_as()
        else:
            if not filename:
                file = self.config_list.currentText() + ".json"
                file = os.path.join(settings.APP_DIR, "configs", file)
            else:
                file = filename
            config = self.config_as_dict()
            with open(file, "w") as f:
                json.dump(config, f)

    @pyqtSlot()
    def save_config_as(self):
        """
        Saves a configuration in a new file.
        """
        f = QFileDialog.getSaveFileName(
            self, "Sélectionner un fichier",
            directory=settings.CONFIG_DIR,
            filter='JSON files (*.json)\nAll files (*)')[0]
        if f is not '':
            if not f.endswith(".json"):
                f = f + ".json"
            logger.info("Saving configuration as {}".format(f))
            self.save_config(f)
            self.list_configs()
            self.update_config(self.config_list.currentIndex())

    @pyqtSlot()
    def send_config(self):
        """
        Send a configuration to the machine.
        """
        self.save_config()
        self.serial_manager.send_config(self.config_as_dict())

    @pyqtSlot()
    def manage_connection(self):
        """
        Manages the connection widgets.
        """
        if self.serial_manager.is_open:
            self.baudrate.setEnabled(True)
            self.serial_ports_list.setEnabled(True)
            self.btn_serial_ports_list.setEnabled(True)
            self.btn_connect.setText("Connecter")
            self.serial_manager.close()
        else:
            port = self.serial_ports_list.currentText()
            baudrate = int(self.baudrate.currentText())
            if self.serial_manager.open(baudrate, port):
                self.baudrate.setEnabled(False)
                self.serial_ports_list.setEnabled(False)
                self.btn_serial_ports_list.setEnabled(False)
                self.btn_connect.setText("Déconnecter")

    @pyqtSlot()
    def choose_file(self):
        """
        Sets the gcode file.
        """
        if not self.file_loaded:
            directory = settings.FILE_DIR
        else:
            directory = os.path.dirname(self.filename.text())
        file = QFileDialog.getOpenFileName(
            self, "Sélectionner un fichier",
            directory=directory,
            filter='GCode files (*.gcode, *.ngc)\nAll files (*)')[0]

        if file is not '':
            self.filename.setText(file)
            self.load_file()

    @pyqtSlot()
    def load_file(self):
        """
        Loads a gcode file.
        """
        file = self.filename.text()
        try:
            logger.info("Loading {}".format(repr(file)))
            with open(file) as f:
                self.code_edit.setText(f.read())
            self.draw_file()
            self.file_loaded = True
        except FileNotFoundError:
            self.choose_file()

    @pyqtSlot()
    def save_file_as(self):
        """
        Saves a gcode file in a nex file.
        """
        if not self.file_loaded:
            directory = settings.FILE_DIR
        else:
            directory = os.path.dirname(self.filename.text())
        file = QFileDialog.getSaveFileName(
            self, "Sélectionner un fichier",
            directory=directory,
            filter='GCode files (*.gcode, *.ngc)\nAll files (*)')[0]
        if file is not '':
            logger.info("Saving {}".format(repr(file)))
            self.filename.setText(file)
            with open(file, 'w') as f:
                f.write(self.code_edit.toPlainText())
            self.file_loaded = True

    @pyqtSlot()
    def save_file(self):
        """
        Saves a gcode file.
        """
        if not self.file_loaded:
            self.save_file_as()
        else:
            file = self.filename.text()
            logger.info("Saving {}".format(repr(file)))
            with open(file, 'w') as f:
                f.write(self.code_edit.toPlainText())

    @pyqtSlot()
    def close_file(self):
        """
        Close the current file.
        """
        self.filename.setText("Pas de fichier.")
        self.code_edit.setText("")
        self.draw_file()

    @pyqtSlot()
    def draw_file(self):
        """
        Draws a gcode file.
        """
        gcode = self.code_edit.toPlainText()
        self.sc.clear()
        current_pos = [0, 0, 0]
        min_x, max_x, min_y, max_y, min_z, max_z = 0, 0, 0, 0, 0, 0

        reverse_x = self.reverse_display_x.isChecked()
        reverse_y = self.reverse_display_y.isChecked()
        reverse_z = self.reverse_display_z.isChecked()

        for n, t in enumerate(parse(gcode)):
            if t['name'] is not 'G':
                continue

            x, y, z = current_pos

            if self.display_draw_steps.isChecked():
                effective_x = x if not reverse_x else -x
                effective_y = y if not reverse_y else -y
                txt = self.sc.addText(str(n))
                txt.setPos(effective_x, effective_y)
                txt.setFlags(QGraphicsItem.ItemIsFocusable | QGraphicsItem.ItemIsMovable |
                             QGraphicsItem.ItemIsSelectable | txt.flags())

            x = t['args'].get('X', x)
            y = t['args'].get('Y', y)
            z = t['args'].get('Z', z)


            effective_z = z if not reverse_z else -z

            p = QPen(QColor((effective_z <= 0) * 255, 0, (effective_z > 0) * 255))
            p.setWidthF(1/self.zoom)
            effective_x_o = current_pos[0] if not reverse_x else -current_pos[0]
            effective_y_o = current_pos[1] if not reverse_y else -current_pos[1]

            if t['value'] in (0, 1):
                effective_x = x if not reverse_x else -x
                effective_y = y if not reverse_y else -y
                s = self.sc.addLine(effective_x_o, effective_y_o, effective_x, effective_y, pen=p)
                s.setFlags(QGraphicsItem.ItemIsFocusable | 
                    QGraphicsItem.ItemIsSelectable | s.flags())
                s.setData(0, t['line'])
            elif t['value'] in (2, 3):
                i = t['args'].get('I', 0)
                j = t['args'].get('J', 0)
                k = t['args'].get('K', 0)

                x_o = current_pos[0]
                y_o = current_pos[1]

                clockwise = (t['value'] == 2)

                logger.debug(
                    "Drawing circle clockwise={clockwise} from {t}".format(**locals()))
                pp = QPainterPath()
                pp.moveTo(effective_x_o,effective_y_o)
                for xc, yc in arc_to_segments((x_o, y_o), (i, j), (x, y), clockwise, 1/self.zoom):
                    effective_x = xc if not reverse_x else -xc
                    effective_y = yc if not reverse_y else -yc
                    min_x = min(min_x, effective_x)
                    max_x = max(max_x, effective_x)
                    min_y = min(min_y, effective_y)
                    max_y = max(max_y, effective_y)
                    pp.lineTo(effective_x,effective_y)

                path_item = self.sc.addPath(pp,pen=p)
                path_item.setFlags(QGraphicsItem.ItemIsFocusable | 
                    QGraphicsItem.ItemIsSelectable | path_item.flags())
                path_item.setData(0, t['line'])

            current_pos = x, y, z

            min_x = min(min_x, current_pos[0] if not reverse_x else -current_pos[0])
            max_x = max(max_x, current_pos[0] if not reverse_x else -current_pos[0])
            min_y = min(min_y, current_pos[1] if not reverse_y else -current_pos[1])
            max_y = max(max_y, current_pos[1] if not reverse_y else -current_pos[1])
            min_z = min(min_z, current_pos[2])
            max_z = max(max_z, current_pos[2])

        self.space_x.display(max_x-min_x)
        self.space_y.display(max_y-min_y)
        self.space_z.display(max_z-min_z)

        if self.display_bounding_box.isChecked():
            p = QPen(QColor(0, 255, 0))
            self.sc.addRect(min_x, min_y, max_x-min_x, max_y-min_y, p)
            txt1 = self.sc.addText(str(max_x-min_x))
            txt1.setPos((min_x+max_x)/2, max_y)
            txt2 = self.sc.addText(str(max_y-min_y))
            txt2.setPos(max_x, (min_y+max_y)/2)

        if self.display_axis.isChecked():
            arrow_length = min(self.space_x.value(), self.space_y.value())
            if reverse_x:
                arrow_length_x = - arrow_length
            else:
                arrow_length_x = arrow_length
            if reverse_y:
                arrow_length_y = - arrow_length
            else:
                arrow_length_y = arrow_length
            
            sign = lambda x: (1, -1)[x < 0]

            x_axis_arrow = QPainterPath()
            x_axis_arrow.lineTo(arrow_length_x,0)
            x_axis_arrow.lineTo(0.95*arrow_length_x,0.05*arrow_length_x)
            x_axis_arrow.moveTo(arrow_length_x,0)
            x_axis_arrow.lineTo(0.95*arrow_length_x,-0.05*arrow_length_x)
            x_axis_arrow.addText(arrow_length_x*1.1, -10 * sign(arrow_length_y), QFont("sans-serif"), "X")
            self.sc.addPath(x_axis_arrow)

            y_axis_arrow = QPainterPath()
            y_axis_arrow.lineTo(0, arrow_length_y)
            y_axis_arrow.lineTo(0.05*arrow_length_y, 0.95*arrow_length_y)
            y_axis_arrow.moveTo(0, arrow_length_y)
            y_axis_arrow.lineTo(-0.05*arrow_length_y, 0.95*arrow_length_y)
            y_axis_arrow.addText(-10 * sign(arrow_length_x), arrow_length_y*1.1, QFont("sans-serif"), "Y")
            self.sc.addPath(y_axis_arrow)

        self.fileview.setScene(self.sc)

    @pyqtSlot()
    def run_preprocessor(self):
        """
        Runs the preprocessor dialog.
        """
        self.preprocessor = PreprocessorDialog(self.code_edit.toPlainText())
        self.preprocessor.accepted.connect(self.end_preprocessor)
        self.preprocessor.show()

    @pyqtSlot()
    def end_preprocessor(self):
        """
        Manages the end of the preprocessing interface.
        """
        self.code_edit.setText(self.preprocessor.gcode)
        self.preprocessor.accepted.disconnect()
        self.draw_file()

    @pyqtSlot()
    def about_license(self):
        """
        Displays informations about the license.
        """
        with open(os.path.join(settings.APP_DIR, 'license_dialog_text')) as f:
            QMessageBox.about(self, "Licence", f.read())

    @pyqtSlot()
    def about_qt(self):
        """
        Displays informations about Qt.
        """
        QMessageBox.aboutQt(self)

    @pyqtSlot()
    def zoomIn(self):
        """
        Zoom the file view in.
        """
        self.fileview.scale(2,2)
        self.zoom *= 2
        self.draw_file()

    @pyqtSlot()
    def zoomOut(self):
        """
        Zoom the file view out.
        """
        self.fileview.scale(1/2,1/2)
        self.zoom *= 1/2
        self.draw_file()

    @pyqtSlot()
    def highlight_selected_path(self):
        """
        Highlight corresponding gcode lines for the selected path.
        """
        sel = self.sc.selectedItems()
        l = []
        for s in sel:
            highlight = QTextEdit.ExtraSelection()
            highlight.cursor = QTextCursor(self.code_edit.document().findBlockByLineNumber(s.data(0)))
            highlight.format.setProperty(QTextFormat.FullWidthSelection, True)
            highlight.format.setBackground(Qt.green)
            l.append(highlight)
        
        if len(l) > 0:
            self.code_edit.setTextCursor(l[0].cursor)
        self.code_edit.setExtraSelections(l)




"""
The serial_manager module
=========================

Provides a class to handle the CNC machine through a serial object.
"""

import serial
import time

from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal

from sivicncdriver.settings import logger

__all__ = ["SerialManager"]

class SerialManager(QObject):
    send_print = pyqtSignal(str, str)

    def __init__(self, serial, fake_mode=False):
        super(SerialManager, self).__init__()
        self.serial = serial
        self.fake_mode = fake_mode
        self.is_open = self.serial.isOpen()

    def open(self, baudrate, serial_port):
        logger.info("Opening {} with baudrate {}".format(repr(serial_port), baudrate))
        self.send_print.emit("Opening {} with baudrate {}".format(repr(serial_port), baudrate), "info")
        self.serial.port = serial_port
        self.serial.baudrate = baudrate
        self.serial.timeout = None
        try :
            self.serial.open()
            self.is_open = True
        except serial.serialutil.SerialException:
            self.is_open = False
            logger.error("Could not open serial port.")
            self.send_print.emit("Could not open serial port.", "error")
            return False
        logger.debug(self.serial.timeout)
        return True

    def close(self):
        logger.info("Closing serial port.")
        self.send_print.emit("Closing serial port.", "info")
        self.serial.close()

    def sendMsg(self, msg):
        if not self.fake_mode and not (self.serial and self.serial.isOpen()):
            self.send_print.emit("Error, no serial port to write.", "error")
            logger.error("Serial manager has no serial opened to send data.")
        elif self.fake_mode:
            self.send_print.emit(msg, "operator")
            logger.info("Sending {} through fake serial port".format(repr(msg)))
        else:
            logger.info("Sending {} through {}".format(repr(msg),self.serial))
            self.send_print.emit(msg, "operator")
            self.serial.write(bytes(msg, encoding='utf8'))

    def waitForConfirm(self):
        if self.fake_mode:
            time.sleep(0.01)
            logger.info("Received {}".format(repr("ok")))
            self.send_print.emit("ok", "machine")
            return True
        elif not (self.serial and self.serial.isOpen()):
            self.send_print.emit("Error, no serial port to read.", "error")
            logger.error("Serial manager has no serial opened to read data.")
            return False

        txt = self.serial.readline().decode('ascii').lower()
        if not "ok" in txt:
            logger.error("Machine could not perform task.")
            self.send_print.emit(txt, "error")
            return False
        else:
            logger.info("Received {}".format(repr(txt)))
            self.send_print.emit("m> {}".format(txt), "machine")
            return True

    def step(self, axis, n):
        self.sendMsg("S0 {}{}\n".format(axis,n))
        return self.waitForConfirm()

    @pyqtSlot(int)
    def step_y(self, n):
        return self.step("Y",n)

    @pyqtSlot(int)
    def step_x(self, n):
        return self.step("X",n)

    @pyqtSlot(int)
    def step_z(self, n):
        return self.step("Z",n)

    def start_continuous(self, axis, direction="forward"):
        if direction == "forward":
            self.sendMsg("S1 {}\n".format(axis))
        else:
            self.sendMsg("S2 {}\n".format(axis))
        return self.waitForConfirm()

    @pyqtSlot()
    def start_continuous_x_forward(self):
        return self.start_continuous("X")

    @pyqtSlot()
    def start_continuous_y_forward(self):
        return self.start_continuous("Y")

    @pyqtSlot()
    def start_continuous_z_forward(self):
        return self.start_continuous("Z")

    @pyqtSlot()
    def start_continuous_x_backward(self):
        return self.start_continuous("X", "backward")

    @pyqtSlot()
    def start_continuous_y_backward(self):
        return self.start_continuous("Y", "backward")

    @pyqtSlot()
    def start_continuous_z_backward(self):
        return self.start_continuous("Z", "backward")

    def stop_continuous(self, axis):
        self.sendMsg("S3 {}\n".format(axis))
        return self.waitForConfirm()

    @pyqtSlot()
    def stop_continuous_x(self):
        return self.stop_continuous("X")

    @pyqtSlot()
    def stop_continuous_y(self):
        return self.stop_continuous("Y")

    @pyqtSlot()
    def stop_continuous_z(self):
        return self.stop_continuous("Z")

    @pyqtSlot()
    def set_origin(self):
        self.sendMsg("G92 X000 Y000 Z000\n")
        return self.waitForConfirm()

    @pyqtSlot()
    def goto_origin(self):
        self.sendMsg("G28\n")
        return self.waitForConfirm()

    @pyqtSlot(dict)
    def send_config(self, d):
        self.sendMsg("S4 X{x_ratio} Y{y_ratio} Z{z_ratio}".format(**d))
        self.waitForConfirm()

        drive = [5, 6, 7]
        self.sendMsg("S{} X".format(drive[d["x_drive"]]))
        self.waitForConfirm()
        self.sendMsg("S{} Y".format(drive[d["y_drive"]]))
        self.waitForConfirm()
        self.sendMsg("S{} Z".format(drive[d["z_drive"]]))
        self.waitForConfirm()

        self.sendMsg("S8 X{x_play} Y{y_play} Z{z_play}".format(**d))
        self.waitForConfirm()

        reverse = {True:9, False:10}
        self.sendMsg("S{} X".format(reverse[d["x_reverse"]]))
        self.waitForConfirm()
        self.sendMsg("S{} Y".format(reverse[d["y_reverse"]]))
        self.waitForConfirm()
        self.sendMsg("S{} Z".format(reverse[d["z_reverse"]]))
        self.waitForConfirm()



import os
import sys
import logging
from pathlib import Path
 
from logging.handlers import RotatingFileHandler


APP_DIR = os.path.dirname(os.path.abspath(__file__))

DEBUG = False

CONFIG_DIR = os.path.join(APP_DIR, "configs", "")

if DEBUG:
    FILE_DIR = os.path.join(APP_DIR, 'gcodes', '')
else:
    FILE_DIR = str(Path.home())

RC_DIR = os.path.join(APP_DIR, "rc")

## Logger stuff
logger = logging.getLogger()

if DEBUG:
    logger.setLevel(logging.DEBUG)
else:
    logger.setLevel(logging.INFO)

formatter = logging.Formatter('%(asctime)s :: %(levelname)s :: %(message)s')

# File log
file_handler = RotatingFileHandler(os.path.join(APP_DIR, 'log.txt'), 'a', 1000000, 1)

if DEBUG:
    file_handler.setLevel(logging.DEBUG)
else:
    file_handler.setLevel(logging.INFO)

file_handler.setFormatter(formatter)
logger.addHandler(file_handler)

if DEBUG:
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.DEBUG)
    stream_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)


"""
The SiviCNCDriver Package
=========================

This is the SiviCNCDriver package. To run the application directly you should
use the ``sivicnc`` command in a shell. Alternately you can use the ``main``
function of the package which doesn't take any parameter.



:Example:

>>> from sivicncdriver.app import main
>>> main()

"""

__version__ = "0.1.2"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'preprocessor_window.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_dialog(object):
    def setupUi(self, dialog):
        dialog.setObjectName("dialog")
        dialog.resize(561, 298)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("traceIcon.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        dialog.setWindowIcon(icon)
        dialog.setModal(True)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(dialog)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.widget_2 = QtWidgets.QWidget(dialog)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.widget = QtWidgets.QWidget(self.widget_2)
        self.widget.setObjectName("widget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.widget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.chk_del_num = QtWidgets.QCheckBox(self.groupBox)
        self.chk_del_num.setChecked(True)
        self.chk_del_num.setObjectName("chk_del_num")
        self.verticalLayout.addWidget(self.chk_del_num)
        self.chk_del_comments = QtWidgets.QCheckBox(self.groupBox)
        self.chk_del_comments.setChecked(True)
        self.chk_del_comments.setObjectName("chk_del_comments")
        self.verticalLayout.addWidget(self.chk_del_comments)
        self.chk_optimize_bounding_box = QtWidgets.QCheckBox(self.groupBox)
        self.chk_optimize_bounding_box.setToolTip("")
        self.chk_optimize_bounding_box.setChecked(True)
        self.chk_optimize_bounding_box.setObjectName("chk_optimize_bounding_box")
        self.verticalLayout.addWidget(self.chk_optimize_bounding_box)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.btn_run_preproc = QtWidgets.QPushButton(self.widget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("rc/work.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_run_preproc.setIcon(icon1)
        self.btn_run_preproc.setObjectName("btn_run_preproc")
        self.verticalLayout_3.addWidget(self.btn_run_preproc)
        self.horizontalLayout.addWidget(self.widget)
        self.groupBox_2 = QtWidgets.QGroupBox(self.widget_2)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.output = QtWidgets.QTextEdit(self.groupBox_2)
        self.output.setObjectName("output")
        self.verticalLayout_2.addWidget(self.output)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.verticalLayout_4.addWidget(self.widget_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(dialog)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_4.addWidget(self.buttonBox)

        self.retranslateUi(dialog)
        QtCore.QMetaObject.connectSlotsByName(dialog)

    def retranslateUi(self, dialog):
        _translate = QtCore.QCoreApplication.translate
        dialog.setWindowTitle(_translate("dialog", "Préprocesseur"))
        self.groupBox.setTitle(_translate("dialog", "Paramètres"))
        self.chk_del_num.setText(_translate("dialog", "Supprimer la numérotation (NXX)"))
        self.chk_del_comments.setText(_translate("dialog", "Supprimer les commentaires"))
        self.chk_optimize_bounding_box.setText(_translate("dialog", "Minimiser l\'aire d\'encadrement"))
        self.btn_run_preproc.setText(_translate("dialog", "Lancer le préprocesseur"))
        self.groupBox_2.setTitle(_translate("dialog", "Sortie"))




import sys

from PyQt5.QtWidgets import QApplication

from sivicncdriver.interface import MainWindow

def main():
    """
    The main function of the application.

    It will create a QApplication and a main window then run the application and
    exit.
    """
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())


import sys
import glob
import serial


def serial_ports():
    """Lists serial ports

    :raises EnvironmentError:
        On unsupported or unknown platforms
    :returns:
        A list of available serial ports
    """
    if sys.platform.startswith('win'):
        ports = ['COM' + str(i + 1) for i in range(256)]

    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this is to exclude your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')

    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')

    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result



"""
A module to parse G-codes.
"""

from sivicncdriver.settings import logger

__all__ = ['parse']

class Stack:
    """
    A Simple LIFO.
    """

    def __init__(self, in_str):
        """
        __init__ method.

        :param in_str: The string which is to be stacked.
        :type in_str: str
        """
        self.in_str = in_str
        self.next = None

    def token(self):
        """
        Yields the elements of the stack.
        """
        while len(self.in_str) > 0:
            yield self.pop()

    def pop(self):
        """
        Pop an element.

        :return: The first element of the string.
        :rtype: str
        """
        r = self.peek()
        self.next = None
        return r

    def peek(self):
        """
        Peek the next element to be popped without popping it.

        :return: The next element
        :rtype: str
        """
        if not self.next:
            self.next = self.in_str[0]
            self.in_str = self.in_str[1:]
        return self.next

SEPARATOR = ('\n', ' ', '(', '=')


def parse_iterator(gcode):
    """
    Function used by the ``parse`` function.

    :param gcode: The gcode which is to be parsed.
    :type gcode: str
    """
    stack = Stack(gcode + '\n')
    line = 0
    for c in stack.token():
        if c is ' ':
            continue
        elif c is '(':
            com = ''
            while stack.peek() is not ')':
                com += stack.pop()
            stack.pop()  # removes the ')'
            yield ('__comment__', com, line)
        elif c is '\n':
            yield ('__next__', '__next__', line)
            line += 1
        else:
            yield (c, parse_value(stack), line)


def parse(gcode):
    """ 
    Parse gcode.

    It yields a dict for each line with :

    name
        name of the code (G, M)
    value
        an integer
    args
        a dict with the code arguments, ex : {'Y':3.0}

    :param gcode: The gcode which is to be parsed.
    :type gcode: str
    """
    name = ""
    value = 0
    args = {}
    for i in parse_iterator(gcode):
        if not i[0] == '__next__':
            if i[0] == '__new_var__':
                name = ''
                value = None
                for a in i[1].keys():
                    OpNode.var[a] = i[1][a]
                    yield {'name': '__new_var__', 'var': a, 'value': i[1][a], 'line':i[2]}
            if i[0] in ('G', 'M'):
                name = i[0]
                value = i[1]
            elif i[0] in 'XYZIJKN':
                args[i[0]] = i[1]
            elif i[0] == '__comment__':
                if name is "":
                    name = 'comment'
                args['comment'] = i[1]
        elif name is not "":
            yield {'name': name, 'value': value, 'args': args, 'line':i[2]}
            name = ""
            value = 0
            args = {}
    yield {'name': name, 'value': value, 'args': args, 'line':i[2]}


def parse_value(stack):
    """
    Parse a value in the stack.
    :param stack: The parser's stack.
    :return: The value.
    """
    r = ''
    while stack.peek() not in SEPARATOR:
        r += stack.pop()
    return float(r)



"""
The arc_calculator module
=========================

It creates small segments from an arc given with G-codes parameters : origin,
end, path to center and sense of rotation.

:Example:

>>> from arc_calculator import arc_to_segments
>>> a = arc_to_segments((0,0),(5,0),(10,0))
>>> for x,y in a:
...     print((x,y))
...
(0.0, 6.12323e-16)
(0.09966, -0.993334)
(0.39469, -1.94708)
(0.87331, -2.8232)
(1.51646, -3.58677)
(2.29848, -4.20735)
(3.1882, -4.66019)
(4.15015, -4.92725)
(5.14598, -4.99787)
(6.136, -4.86924)
(7.08072, -4.54649)
(7.94249, -4.04249)
(8.68696, -3.37733)
(9.28444, -2.57752)
(9.71111, -1.67495)
(10, 0)

"""
from math import atan2, cos, sin, sqrt, pi
from decimal import *

from sivicncdriver.settings import logger


getcontext().prec = 6


def arc_to_segments(start, vect_to_center, end, clockwise=False, length=1,):
    """
    Creates small segments from an arc.

    Uses Decimal for better precision. It yields the vertices.

    :param start: The starting position
    :param vect_to_center: A vector to go to the center of the arc from the
        starting position
    :param end: The ending position
    :param clockwise: Should it go clockwise ?
    :param length: length of the segments 
    :type start: A float tuple
    :type vect_to_center: A float tuple
    :type end: A float tuple
    :type clockwise: bool
    :type length: float
    :return: None, it yields vertices.
    """

    v = Decimal(vect_to_center[0]), Decimal(vect_to_center[1])
    e = Decimal(end[0]), Decimal(end[1])
    s = Decimal(start[0]), Decimal(start[1])
    radius = (v[0]**2 + v[1]**2).sqrt()
    start_angle = Decimal(atan2(-v[1], -v[0])) % Decimal(2*pi)
    center = (s[0]+v[0], s[1]+v[1])
    end_angle = Decimal(atan2(e[1]-center[1], e[0]-center[0])) 

    if clockwise:
        arc_length = -Decimal(2*pi) + (end_angle - start_angle)
    else:
        arc_length = (end_angle - start_angle)

    arc_length = arc_length % Decimal(2*pi)
    if arc_length < 0 and not clockwise:
        arc_length += Decimal(2*pi)
    elif arc_length > 0 and clockwise:
        arc_length -= Decimal(2*pi)
    nb_step = int(abs(arc_length*radius/Decimal(length)))
    
    logger.debug("Arc to segments : start={s}, end={e}, v={v}".format(**locals()))
    logger.debug("Arc to segments : start_angle={start_angle}, end_angle={end_angle}, radius={radius}, arc_length={arc_length}, nb_step={nb_step}".format(**locals()))
    logger.debug(arc_length * radius)
    if abs(arc_length * radius) < length:
        yield start
        yield end
    else:
        d_angle = arc_length / nb_step
        angle = start_angle
        for _ in range(nb_step):
            yield (
                    float(center[0] + radius*Decimal(cos(angle))), 
                    float(center[1] + radius*Decimal(sin(angle)))
                  )
            angle += d_angle
        yield end



from setuptools import setup, find_packages
import sivicncdriver

setup(name='SiviCNCDriver',
      version=sivicncdriver.__version__,
      description='A software to control my CNC',
      long_description=open('README.rst').read(),
      url='http://github.com/klafyvel/SiviCNCDriver',
      author='klafyvel',
      author_email="sivigik@gmail.com",
      license='GPL',
      include_package_data=True,
      packages=find_packages(),
      install_requires= open('requirements.txt').read().split('\n'),
      classifier=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Manufacturing',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: French',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
      ],
      entry_points = {
        'gui_scripts': [
            'sivicnc = sivicncdriver.__main__:main',
        ],
    },
)



"""
The preprocessor module
=======================

Provides the PreprocessorDialog class.
"""
import os

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

from sivicncdriver.preprocessor_window import Ui_dialog
from sivicncdriver import settings
from sivicncdriver.settings import logger
from sivicncdriver.gcode import parse
from sivicncdriver.arc_calculator import arc_to_segments


class PreprocessorDialog(QDialog, Ui_dialog):
    """
    The preprocessor dialog.
    """
    def __init__(self, gcode, parent=None):
        """
        The __init__ method.
        :param gcode: The gcode which is to be preprocessed.
        :param parent: Qt stuff.
        :type gcode: str
        """
        super(PreprocessorDialog, self).__init__(parent=parent)
        self.setupUi(self)
        self.gcode = gcode
        self.original = gcode
        self.has_been_run_once = False

        icon1 = QIcon()
        icon1.addPixmap(QPixmap(os.path.join(settings.RC_DIR, "work.png")), QIcon.Normal, QIcon.Off)
        self.btn_run_preproc.setIcon(icon1)

        self.btn_run_preproc.clicked.connect(self.run_preprocessor)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.cancel)

    @pyqtSlot()
    def run_preprocessor(self):
        """
        Runs the preprocessor on the G-Code.
        """
        self.remove_useless()
        self.has_been_run_once = True

    def remove_useless(self):
        """
        Remove useless things in the code according to the UI options.
        """
        rm_nums = self.chk_del_num.isChecked()
        rm_comm = self.chk_del_comments.isChecked()
        minimize = self.chk_optimize_bounding_box.isChecked()
        if minimize:
            x,y = self.get_minimize_bounding_box()
        r = ''
        last_g00 = None
        line = 0
        for i in parse(self.gcode):
            if 'M' in i['name'] or 'G' in i['name']:
                if not rm_nums and 'N' in i['args']:
                    r += 'N' + str(int(i['args']['N'])) + ' '
                r += i['name'] + str(int(i['value'])) + ' '
                for a in i['args']:
                    if (not minimize and a in 'XY') or (a in 'ZIJKF'):
                        r += a + str(i['args'][a]) + ' '
                    elif minimize and a =='X':
                        r += a + str(i['args'][a]-x) + ' '
                    elif minimize and a == 'Y':
                        r += a + str(i['args'][a]-y) + ' '
                if i['name'] == 'G' and i['value'] == 0:
                    last_g00 = line
                r += '\n'
                line += 1
            if 'comment' in i['args'] and not rm_comm:
                logger.debug(i)
                r += '(' + i['args']['comment'] + ')\n'
                line += 1
        if last_g00:
            r = r.split('\n')
            r[last_g00] = 'G00 X0.0000 Y0.0000'
            r = '\n'.join(r)
        self.output.setText(r)
        self.gcode = r

    def get_minimize_bounding_box(self):
        """
        Computes a new origin for the drawing.
        """
        min_x, min_y = 0,0
        set_x, set_y = False, False
        x_o, y_o = 0,0
        x,y = 0,0
        z = 0
        r = ''
        for t in parse(self.gcode):
            if t['name'] is not 'G':
                continue
            if 'Z' in t['args']:
                z = t['args']['Z']
            logger.debug("Z = {}".format(z))
            if z > 0 :
                continue
            logger.debug("Obviously, z<=0.")
            if 'X' in t['args']:
                x = t['args']['X']
                if not set_x :
                    min_x = x
                    set_x = True
                min_x = min(min_x, x)
            if 'Y' in t['args']:
                y = t['args']['Y']
                if not set_y :
                    min_y = y
                    set_y = True
                min_y = min(min_y, y)

            if t['value'] in (2,3):
                i = t['args'].get('I', 0)
                j = t['args'].get('J', 0)
                k = t['args'].get('K', 0)
                clockwise = (t['value'] == 2)
                for xc, yc in arc_to_segments((x_o, y_o), (i,j), (x,y), clockwise):
                    min_x = min(min_x, xc)
                    min_y = min(min_y, yc)
            x_o, y_o = x,y
        logger.debug("Minimizing by setting origin to {},{}".format(x,y))
        return min_x, min_y


    @pyqtSlot()
    def accept(self):
        if not self.has_been_run_once:
            self.run_preprocessor()
        super(PreprocessorDialog, self).accept()

    @pyqtSlot()
    def cancel(self):
        self.gcode = self.original
        self.reject()
